#include <dpf-internal.h>

static void dump(Atom p, int n);

static void indent(int n) {
        while(n-- > 0)
                putchar(' ');
}

/* Print out the tree. */
void dpf_output(void) {
	if(!dpf_base->kids.lh_first) {
		printf("Nil trie\n");
		return;
	}
	printf("\nDumping ir.\n");
	printf("base: \n");
	v_dump(dpf_iptr);
	printf("rest: \n");
	dump(dpf_base->kids.lh_first, 3);
}

static void dump_eqnode(Atom p, int n, int bits) {
	struct ir *ir;

	/* have to handle hash table as well. */
	ir = &p->ir;

	if(!p->ht) {
		indent(n);
		printf("msg[%d:%d] & 0x%x == %d, refcnt = %d, level = %d", 
			ir->u.eq.offset, bits, ir->u.eq.mask, ir->u.eq.val, p->refcnt, ir->level);
		if(p->pid)
			printf(", pid = %d\n", p->pid);
		printf("\n");
		v_dump(p->code);
		dump(p->kids.lh_first, n + 5);
	} else {
		Atom hte, h;
		int i, sz;
		Ht ht;

		printf("ht: \n");
		v_dump(p->code);
		for(ht = p->ht, i = 0, sz = ht->htsz; i < sz; i++) {
			if(!(hte = ht->ht[i]))
				continue;

			indent(n);

			printf("{ val = %d, ref = %d", hte->ir.u.eq.val, hte->refcnt);
			if(hte->pid)
				printf(", pid = %d", hte->pid);
			printf(" }\n");
			dump(hte->kids.lh_first, n + 5);
			v_dump(hte->code);

			for(h = hte->next; h; h = h->next) {
				indent(n);
				printf("-> { val = %d, ref = %d", h->ir.u.eq.val, h->refcnt);
				if(h->pid)
					printf(", pid = %d", h->pid);
				printf("}\n");
				dump(h->kids.lh_first, n + 5);
				v_dump(h->code);
			}
		}
		indent(n);
		printf(" ==  msg[%d:%d] & 0x%x,  refcnt = %d, level = %d\n", 
			ir->u.eq.offset, bits, ir->u.eq.mask, p->refcnt,
ir->level);
	}

}

static void dump_shiftnode(Atom p, int n, int bits) {
	struct ir *ir;

	indent(n);
	ir = &p->ir;
	printf("msg + msg[%d:%d] & 0x%x << %d, align = %d, refcnt = %d, level = %d", 
		ir->u.shift.offset, bits, ir->u.shift.mask, ir->u.shift.shift, ir->u.shift.align, p->refcnt, ir->level);
	if(p->pid)
		printf(", pid = %d\n", p->pid);
	printf("\n");
	v_dump(p->code);
	dump(p->kids.lh_first, n + 5);
}

static void dump(Atom p, int n) {
	if(!p) return;

	for(;; p = p->sibs.le_next) {

		if(iseq(&p->ir))
			dump_eqnode(p, n, p->ir.u.eq.nbits);
		else  {
			demand(isshift(&p->ir), bogus op);
			dump_shiftnode(p, n, p->ir.u.shift.nbits);
		}

		if(!p->sibs.le_next) 
			return;

		indent(n - 2);
		printf("OR\n");
	}
}
