#include <dpf.h>
#include <old-dpf.h>
#include <dpf-internal.h>
#include <demand.h>

static struct dpf_frozen_ir df_connected[] = {
        /*      op              len     r0      r1      imm     */
        {       DPF_EQI,        2,      1,      0,      0x8}, /* 0 */
        {       DPF_BITS16I,    0,      0,      0,      0xc}, /* 1 */
        {       DPF_EQI,        2,      1,      0,      0x6}, /* 2 */
        {       DPF_BITS8I,     0,      0,      0,      0x17}, /* 3 */
        {       DPF_EQI,        2,      1,      0,      0x1a12}, /* 4 */
        {       DPF_BITS16I,    0,      0,      0,      0x1a}, /* 5 */
        {       DPF_EQI,        2,      1,      0,      0x900}, /* 6 */
        {       DPF_BITS16I,    0,      0,      0,      0x1c}, /* 7 */
        {       DPF_EQI,        2,      1,      0,      0x9019}, /* 8 */
        {       DPF_BITS16I,    0,      0,      0,      0x22}, /* 9 */
        {       DPF_EQI,        2,      1,      0,      0x50d}, /* 10 */
        {       DPF_BITS16I,    0,      0,      0,      0x24}, /* 11 */
};


static struct dpf_frozen_ir df_listen[] = {
        /*      op              len     r0      r1      imm     */
        {       DPF_EQI,        2,      1,      0,      0x8}, /* 0 */
        {       DPF_BITS16I,    0,      0,      0,      0xc}, /* 1 */
        {       DPF_EQI,        2,      1,      0,      0x6}, /* 2 */
        {       DPF_BITS8I,     0,      0,      0,      0x17}, /* 3 */
        {       DPF_EQI,        2,      1,      0,      0x50d}, /* 4 */
        {       DPF_BITS16I,    0,      0,      0,      0x24}, /* 5 */
};


/*
 * Get filter; if dst_port is zero, we take the filter that accepts packets
 * from any destination.
 */
void *mk_tcp(int *sz, int src_port, int dst_port, short *src_ip) 
{
    if (src_port == 0) {
        *sz = sizeof df_listen / sizeof df_listen[0];
        df_listen[4].imm = dst_port;
        return (void *)df_listen;
    } else  {
        *sz = sizeof df_connected / sizeof df_connected[0];
        df_connected[4].imm = src_ip[0] & 0xFFFF;
        df_connected[6].imm = src_ip[1] & 0xFFFF;
        df_connected[8].imm = src_port;
        df_connected[10].imm = dst_port;

        return (void *)df_connected;
    }
}

int main() {
        int     sz, pid, pid2, pid3;
        struct dpf_ir *filter;
	short ip[] = { 0x1010, 0x2020 };
	unsigned msg[1024];
	unsigned msg2[1024];
	unsigned msg3[1024];

#define WRITE(type, m, offset, val) do { 	\
	type tmp = val;				\
	memcpy((char *)m + offset, &tmp, sizeof tmp);	\
} while(0)


	WRITE(unsigned short, msg, 12, 8);
	WRITE(unsigned char, msg, 23, 6);
	WRITE(unsigned, msg, 26, 538972176);
	WRITE(unsigned, msg, 34, 283182290);

	/* Create long filter, make sure it is accepted. */
        filter = dpf_xlate(mk_tcp(&sz, 1234, 4321, ip), sz);

        if ((pid = dpf_insert(filter)) < 0)
        	demand(0, bogus insertion);
	dpf_output();

	if(pid != dpf_iptr((void *)msg, sizeof msg))
		fatal(did not recognize the message);

	/* Create a short listen filter on another port. */
        filter = dpf_xlate(mk_tcp(&sz, 0, 1111, ip), sz);

        if ((pid2 = dpf_insert(filter)) < 0)
        	demand(0, bogus insertion);

	WRITE(unsigned short, msg2, 12, 8);
	WRITE(unsigned char, msg2, 23, 6);
	WRITE(unsigned short, msg2, 36, 1111);
	dpf_output();

	if(pid != dpf_iptr((void *)msg, sizeof msg))
		fatal(did not recognize the message);
	if(pid2 != dpf_iptr((void *)msg2, sizeof msg))
		fatal(did not recognize the message);


	/* now create a long connected filter. */
        filter = dpf_xlate(mk_tcp(&sz, 1212, 1111, ip), sz);

        if ((pid3 = dpf_insert(filter)) < 0)
        	demand(0, bogus insertion);

	dpf_output();

	return 0;
}
