#include <stdio.h>
#include <dpf.h>
#include <stdlib.h>
#include "old-dpf.h"
#include "stopwatch.h"
#include "dpf_lib.h"
#include "demand.h"

extern void *mk_pathfinder(int *sz, int src_port, int dst_port);

static unsigned short path[22] =
{
        0,      /* 0 */
        0,      /* 1 */
        0xed,   /* 2 */
        0x20,   /* 3 */
        (6 << 8), /* 4 */
        0,      /* 5 */
        0x4501, /* 6 */
         0xc00c,        /* 7 */
        0,      /* 8 */
        0,      /* 9 */
        1234,   /* 10 */
        4321    /* 11 */
};

static void insert_trials(char *msg, int n, void *filter) {
	int i, pid;
	double time;

	printf("%s, %d trials\n", msg, n);

	startwatch(0);
	for (i = 0; i < n; i++) {
		if ((pid = dpf_insert(filter)) < 0) 
			fatal(bogsu insert);
		if (dpf_delete(pid) < 0)
			fatal(bogus delete);
	}
	time = stopwatch(0);

	printf("each insertion cost %f microseconds\n", 
		time / n * 1000 * 1000);
}


static void demux_trials(int n, void *filter, int verbose) {
	int i,pid;
	double time;

	printf("classification overhead, %d trials\n", n);
	if ((pid = dpf_insert(filter)) < 0)
		fatal(bogus insertion);
	if(pid != dpf_iptr((void *)path, sizeof path))
		fatal(bogus classification);
	if(verbose) dpf_output();
	startwatch(0);
	for (i = 0; i < n / 16; i++) {
		/* 4 */
		dpf_iptr((void *)path, sizeof path);
		dpf_iptr((void *)path, sizeof path);
		dpf_iptr((void *)path, sizeof path);
		dpf_iptr((void *)path, sizeof path);
		/* 4 */
		dpf_iptr((void *)path, sizeof path);
		dpf_iptr((void *)path, sizeof path);
		dpf_iptr((void *)path, sizeof path);
		dpf_iptr((void *)path, sizeof path);
		/* 4 */
		dpf_iptr((void *)path, sizeof path);
		dpf_iptr((void *)path, sizeof path);
		dpf_iptr((void *)path, sizeof path);
		dpf_iptr((void *)path, sizeof path);
		/* 4 */
		dpf_iptr((void *)path, sizeof path);
		dpf_iptr((void *)path, sizeof path);
		dpf_iptr((void *)path, sizeof path);
		dpf_iptr((void *)path, sizeof path);
	}

	time = stopwatch(0);
	printf("each classification cost %f microseconds\n", 
		time / n * 1000 * 1000);
}

extern void * dpf_xlate(void *, int);
extern void dpf_output(void);

int main(int argc, char *argv[]) {
	int     i,n, sz, pid, new, v;
	struct dpf_ir *filter;

	/* get options */
	for (v = 0, new = i = 1; i < argc; i++)
		if (strcmp(argv[i], "-n") == 0)
			n = atoi(argv[++i]);
		else if (strcmp(argv[i], "-p") == 0)
			new = 1;
		else if (strcmp(argv[i], "-v") == 0)
			v = 1;
		else if (strcmp(argv[i], "-e") == 0) 
			new = 0;
		else {
			puts("Options:");
			puts("\t-n #: number of timed trials.");
			puts("\t-p: new protocol insertion.\n");
			puts("\t-e: existing protocol insertion.\n");
			return 1;
		}

	filter = (void*)dpf_xlate(mk_pathfinder(&sz, 1234, 4321), sz);

	if ((pid = dpf_insert(filter)) < 0)
		demand(0, bogus insertion);
	if(dpf_iptr((void *)path, sizeof path) != pid) 
		fatal(bogus classification);
	if (dpf_delete(pid) < 0)
		demand(0, bogus insertion);

	if(new) {
		insert_trials( "inserting a single filter", n, filter);
		demux_trials(n * n, filter, v);
	} else {

		/* create the hash table. */
		if(dpf_insert(dpf_xlate(mk_pathfinder(&sz, 1234, 4322), sz))<0)
			fatal(Bogus insertion);
		if(dpf_insert(dpf_xlate(mk_pathfinder(&sz, 1234, 4323), sz))<0)
			fatal(Bogus insertion);

		filter = dpf_xlate(mk_pathfinder(&sz, 1234, 4321), sz);

		if ((pid = dpf_insert(filter)) < 0)
			demand(0, bogus insertion);
		if(dpf_iptr((void *)path, sizeof path) != pid) 
			fatal(bogus classification);
		if (dpf_delete(pid) < 0)
			demand(0, bogus insertion);

	
		insert_trials("inserting filter in existing protocol family",
		    		n, filter);
		demux_trials(n * n, filter, v);
	}
	return 0;
}
